#################################################################################
# Created       : Martin Foster
# Modified      : 26/01/2003
#################################################################################
#
# Pull - Post retrieval system
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Pull;

use Cwd;							# Current working directory
use Archive::Zip qw(:ERROR_CODES :CONSTANTS);			# Zip compression library
use Ethereal::Date;						# Date handler
use Ethereal::Mail;						# Mail handleR


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Posts
#
# Allows a host to retreive the posts for a room.  This will fetch all of the
# posts currently in the database, then output the file using a normal GNU
# ZIP format.

sub Posts
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $choice   = shift;					# Choice of the matter

	my $date;						# Date handler
	my $data     = $database->{'HANDLE'};			# Database handle
	my $send;						# Mail sender
	my $zip;						# Compression file handle

	my $statement;						# Database statement

	my $formatting;						# Formatting of message
	my $mail;						# Mailed message
	my $message;						# Message to append
	my $notice;						# Notice to send
	my $post;						# Retreived post
	my $seperator;						# Seperator bar

	my $cwd;						# Current working directory
	my $stamp;						# Retreived date
	my $time;						# Specific time

	my $html;						# HTML file
	my $status;						# Zipped status
	my $zipped;						# Zipped file

	my %system;						# System hash


	####################
	# Program Area

	# Make sure output is necessary
	if ((defined($param->{'GETPOSTS'})) && (defined($param->{'ROOM'})))
	{
		# Retreive hash
		$database->GetHashSystem(\%system);


		# Create instance
		$date = new Ethereal::Date();


		# Retreive Email details
		($message)   = $database->DocumentGetMailPosts();

		# Retreive seperator bar
		($seperator) = $database->FetchRealmSeperator($param->{'ROOM'});


		# Administrative and supervisory
		if (defined($choice) && ($choice > 0))
		{
			# Is user an admin
			($admin) = $database->DataGetAdmin($param->{'USER'});
			($mail)  = $database->DataGetPuppeteerEmail($param->{'USER'});

			# Prepare
			if ($admin eq 'yes')
			{
				# Administrators
				$statement = $data->prepare("SELECT PostPartialFormat
					FROM  Post
					WHERE RealmName=?
					  AND PostTo != ?
					  AND PostClass != ?
					ORDER BY PostIDNumber");
				$statement->execute($param->{'ROOM'}, 'system', 'duplicate');
			}
			else
			{
				# Supervisors
				$statement = $data->prepare("SELECT PostPartialFormat
					FROM  Post
					WHERE RealmName=?
					  AND PostTo=?
					ORDER BY PostIDNumber");
				$statement->execute($param->{'ROOM'}, 'all');
			}
		}

		# Automated archiving
		elsif (defined($choice) && ($choice < 1))
		{
			# Pull address
			$mail = shift; 

			# Regulate date/time search
			$timestamp =  time;
			$timestamp -= $system{'SetTimeDay'};

			# Define query
			$statement = $data->prepare("SELECT PostFullFormat
				FROM  Post
				WHERE RealmName=?
  				  AND PostTimestamp > ?
				  AND PostTo = ?
				ORDER BY PostIDNumber");
			
			# Execute statement
			$statement->execute($param->{'ROOM'}, $timestamp, 'all');
		}


		# Default post retreival
		else
		{
			# Return if root undefined
			unless (defined($param->{'ROOM'}))
			{ return 0; }

  
			# Pull Email address
			($mail) = $database->DataGetPuppeteerEmail($param->{'USER'});


			# Fake post
			# Retreive 
			($notice) = $database->FetchRealmNotice($param->{'ROOM'});

			# Adapt notice
			$notice =~ s/MEMAIL/$mail/gs;

			# Make post
			$data->do("INSERT INTO Post
				(RealmName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES (?,?,?,?,?,?)", {},

			$param->{'ROOM'},
			$param->{'USER'},
			time,
			$notice,
			$notice,
			$notice);



			# Determin field to use
			# Retreive formatting used
			($formatting) = $database->DataGetFormatting($param->{'USER'});

			# Full formatting
			if ($formatting eq 'complete')
			{ $formatting = 'PostFullFormat'; }

			# Full less images
			elsif ($formatting eq 'imageless')
			{ $formatting = 'PostImagelessFormat'; }

			# Spartan formatting
			elsif ($formatting eq 'spartan')
			{ $formatting = 'PostPartialFormat'; }


			# Define query
			$statement = $data->prepare("SELECT $formatting
				FROM  Post
				WHERE RealmName=?
  				  AND PostIDNumber > ?
  				  AND (PostTo=?
				   OR PostTo=?)
				ORDER BY PostIDNumber");

			
			# Execute statement
			$statement->execute($param->{'ROOM'}, $param->{'ROOT'}, $param->{'USER'}, 'all');
		}

		# Assign timestamp
		$stamp = $date->GetDateShort(time);
		$time  = $date->GetDateFile(time);

		# Variables
		my $room = lc("$param->{USER}_$param->{ROOM}");

		# Replaces non printable characters with
		$room =~ s/\W+//gs;

		# No longer then twenty 
		$room = substr($room, 0, 20);

		# HTML title
		$html = "${room}_${time}";


		# Determine official filenames
		$zipped = "${html}.zip";
		$html   = "${html}.html";


		# Retreieve current directory
		$cwd = cwd;

		# Change directory
		chdir($system{'LocTemp'})
		 or die("Cannot change to directory: $!"); 


		# Open filehandle
		open(HTML, ">${html}")
		 or die "Cannot open file: $!\n";
    

		# Display
		# HTML header 
		print HTML $database->FetchRealmHeader($room), "\n";

		# Cycle through posts
		while ($post = $statement->fetchrow())
		{
			# Add to HTML file
			print HTML $post, "\n";
			print HTML $seperator, "\n";
		}

		# HTML footer
		print HTML $database->FetchRealmFooter($room), "\n";

		# Finish Query
		$statement->finish();

		# Close file
		close(HTML);


		# Compress file
		# Create instance
		$zip = Archive::Zip->new();

		# Add file
		$zip->addFile($html)
		 or die("Cannot add file: $!");

		# Compress
		$status = $zip->writeToFileNamed($zipped);

		# Verify
		die("Cannot Compress File: $status") if ($status ne AZ_OK);


		# Change directory
		chdir($cwd)
		 or die("Cannot change to directory: $!"); 


		# Mail sending
		# Initiate handles
		$send = tie(*MAIL, 'Ethereal::Mail', $database);

		# Initial setup
		$send->ToggleWrap();
		$send->SetSubject("Post extract for $param->{ROOM}");
		$send->SetSearch(
			REALM => $param->{'ROOM'},
			DATE  => $timestamp
		 );

		# Recipients
		$send->AddTo($mail);

		# Attachments
		$send->AddAttach(
			Path     => "$system{LocTemp}/$zipped",
			Type     => "application/x-zip",
			Encoding => "base64"
		 );


		# Message
		print MAIL $message;

		# Close and send
		close(MAIL);


		# Unlink
		unlink("$system{LocTemp}/$html");
		unlink("$system{LocTemp}/$zipped");

		
		# Cleanup
		delete $param->{'GETPOSTS'};
	}
}

# Alias for automated posts
sub AutomatedPosts
{ Posts(shift, shift, shift, shift, 0, shift); }

# Alias for administrative posts
sub AdminPosts
{ Posts(shift, shift, shift, shift, 1); }

1;								# Return True
